==> _private_chat-0.0.1.tar.gz: lib/call/widgets/video_call_page.dart <==
import 'dart:async';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:agora_rtc_engine/agora_rtc_engine.dart';
import 'package:get/get.dart';
import 'package:_private_chat/utils/utils.dart';
import 'package:_private_chat/widgets/widget_timer.dart';
import 'package:_private_core/_private_core.dart';
import '../call_helper.dart';
import 'widget_call_coming.dart';

enum VideoCallEventStatus { incomingCall, calling }

class WidgetVideoCall extends StatefulWidget {
  final VideoCallEventStatus status;

  final Call? call;

  final String? receiverId;
  final String? receiverName;
  final String? receiverPic;
  final String? receiverUserType;
  final String? conversationId;
  final String? conversationType;

  const WidgetVideoCall({
    Key? key,
    required this.status,
    this.call,
    this.receiverId,
    this.receiverName,
    this.receiverPic,
    this.receiverUserType,
    this.conversationId,
    this.conversationType,
  }) : super(key: key);

  @override
  WidgetVideoCallState createState() => WidgetVideoCallState();
}

class WidgetVideoCallState extends State<WidgetVideoCall> {
  late final RtcEngine _engine = createAgoraRtcEngine();
  Call? call;
  bool isInitialized = false;
  bool muted = false;
  bool _isAlreadyEnd = false;
  final _users = <int>[];
  Map<String, dynamic> state = {};

  late VideoCallEventStatus status = widget.status;

  String get nameOnTopbar =>
      '${isCallComing ? call!.callerName : call!.receiverName}';

  bool get isCallComing => call?.callerId != loggedFirebaseId;

  @override
  void initState() {
    super.initState();
    _initialize();
  }

  Future<void> _initialize() async {
    setState(() => isInitialized = false);
    if (status == VideoCallEventStatus.incomingCall) {
      if (widget.call == null) {
        Get.back();
        return;
      }
      call = widget.call!;
    } else {
      call = await CallHelper.dial(
        receiverName: widget.receiverName ?? '',
        receiverId: widget.receiverId ?? '',
        receiverPic: widget.receiverPic,
        receiverUserType: widget.receiverUserType ?? '',
        data: {
          'conversationId': widget.conversationId,
          'conversationType': widget.conversationType,
        },
      );
      if (call == null) {
        throw Exception('Call infos is null!');
      }
      await _engine.initialize(RtcEngineContext(
        appId: call!.agoraAppId!,
      ));

      appDebugPrint('Call info: ${call!.toJson()}');

      _engine.registerEventHandler(RtcEngineEventHandler(
        onError: (ErrorCodeType err, String msg) {
          appDebugPrint('[EventHandler] onError: $msg');
          if (!isCallComing) {}
        },
        onJoinChannelSuccess: (RtcConnection connection, int elapsed) {
          appDebugPrint(
              '[EventHandler] onJoinChannel: ${connection.channelId}, uid: ${connection.localUid}');
          if (!isCallComing) {}
        },
        onLeaveChannel: (RtcConnection connection, RtcStats stats) {
          appDebugPrint('[EventHandler] onLeaveChannel: $stats');
          _users.clear();
        },
        onUserJoined: (RtcConnection connection, int remoteUid, int elapsed) {
          appDebugPrint('[EventHandler] userJoined: $remoteUid');
          setState(() {
            _users.add(remoteUid);
          });
          if (!isCallComing) {
            CallHelper.updateCallState(call!.channelId,
                status: CS_PICKED,
                pickedAt: DateTime.now().millisecondsSinceEpoch);
          }
        },
        onUserOffline: (RtcConnection connection, int remoteUid,
            UserOfflineReasonType reason) {
          appDebugPrint('[EventHandler] userOffline: $remoteUid');
          setState(() {
            _users.remove(remoteUid);
          });
          Get.back();
          // if (isalreadyendedcall == false) {
          //   CallHelper.onEndCall(widget.call);
          // }
        },
        onFirstRemoteVideoFrame: (RtcConnection connection, int remoteUid,
            int width, int height, int elapsed) {
          appDebugPrint(
              '[EventHandler] firstRemoteVideo: $remoteUid ${width}x $height');
        },
      ));

      await _engine.enableVideo();
      // await _engine.setVideoEncoderConfiguration(
      //   const VideoEncoderConfiguration(
      //     dimensions: VideoDimensions(width: 640, height: 360),
      //     frameRate: 15,
      //     bitrate: 0,
      //   ),
      // );
      await _engine.startPreview();

      await _engine.joinChannel(
        token: call!.channelToken!,
        channelId: call!.channelId!,
        uid: 0,
        options: const ChannelMediaOptions(
          channelProfile: ChannelProfileType.channelProfileLiveBroadcasting,
          clientRoleType: ClientRoleType.clientRoleBroadcaster,
        ),
      );
    }
    setState(() => isInitialized = true);
  }

  void _onToggleMute() {
    setState(() => muted = !muted);
    _engine.muteLocalAudioStream(muted);
  }

  void _onEnd() {
    if (_isAlreadyEnd) {
    } else if (state[DB_KEY_STATUS] == CS_CANCELLED) {
    } else if (state[DB_KEY_STATUS] != CS_PICKED &&
        state[DB_KEY_STATUS] != CS_ENDED) {
      CallHelper.endCall(call: call!, status: CS_CANCELLED);
    } else {
      CallHelper.endCall(call: call!, status: CS_ENDED);
    }
    _users.clear();
    _engine.leaveChannel();
    _engine.release();
    _isAlreadyEnd = true;
  }

  @override
  void dispose() {
    _onEnd();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      color: Colors.transparent,
      child: Container(
        margin: const EdgeInsets.all(16),
        clipBehavior: Clip.hardEdge,
        padding: const EdgeInsets.all(5),
        decoration: BoxDecoration(
            color: hexColor('#E38F3E'),
            borderRadius: BorderRadius.circular(16)),
        child: !isInitialized
            ? Container(
                decoration: BoxDecoration(
                    color: Colors.white,
                    borderRadius: BorderRadius.circular(16)),
                child: const Center(child: CupertinoActivityIndicator()))
            : StreamBuilder<DocumentSnapshot>(
                stream: CallHelper.callStateStream(call!.channelId),
                builder: (_, snapshot) {
                  if (snapshot.data == null || snapshot.data!.exists == false) {
                    return const Center(child: CupertinoActivityIndicator());
                  }
                  Map<String, dynamic> doc =
                      snapshot.data!.data() as Map<String, dynamic>;
                  state = doc;
                  // appDebugPrint(
                  //     '[EventHandler] doc: ${call!.eventUniqueId} $doc');

                  if (doc[DB_KEY_STATUS] == CS_ENDED) {
                    Get.back();
                  } else if (doc[DB_KEY_STATUS] == CS_CANCELLED &&
                      isCallComing) {
                    Get.back();
                  } else if (doc[DB_KEY_STATUS] == CS_REJECTED &&
                      !isCallComing) {
                    Get.back();
                  }

                  if (doc[DB_KEY_STATUS] == CS_CALLING && isCallComing) {
                    return WidgetCallComming(
                      name: nameOnTopbar,
                      onCancel: () {
                        Get.back();
                        CallHelper.endCall(call: call!, status: CS_REJECTED);
                      },
                      onAccept: () async {
                        // bool isgranted = await CallHelper.permissions();
                        // if (isgranted) {
                        setState(() {
                          status = VideoCallEventStatus.calling;
                        });
                        _initialize();
                        // } else {
                        //   Get.back();
                        //   CallHelper.endCall(call: call!, status: CS_REJECTED);
                        // }
                      },
                    );
                  }
                  return Column(
                    children: [
                      Expanded(
                        child: Stack(
                          alignment: Alignment.topCenter,
                          children: [
                            if (_users.isNotEmpty &&
                                doc[DB_KEY_STATUS] == CS_PICKED)
                              ClipRRect(
                                borderRadius: BorderRadius.circular(16),
                                child: AgoraVideoView(
                                  controller: VideoViewController.remote(
                                    rtcEngine: _engine,
                                    canvas: VideoCanvas(uid: _users.first),
                                    connection: RtcConnection(
                                        channelId: call!.channelId!),
                                    useFlutterTexture: true,
                                    useAndroidSurfaceView: false,
                                  ),
                                ),
                              )
                            else
                              Container(
                                decoration: BoxDecoration(
                                    color: Colors.white,
                                    borderRadius: BorderRadius.circular(16)),
                                width: Get.width,
                                height: Get.height,
                              ),
                            Positioned(
                              bottom: 20,
                              right: 20,
                              child: SizedBox(
                                width: Get.width / 8,
                                child: AspectRatio(
                                  aspectRatio: 1.5,
                                  child: ClipRRect(
                                    borderRadius: BorderRadius.circular(8),
                                    child: AgoraVideoView(
                                      controller: VideoViewController(
                                        rtcEngine: _engine,
                                        canvas: const VideoCanvas(uid: 0),
                                        useFlutterTexture: true,
                                        useAndroidSurfaceView: false,
                                      ),
                                    ),
                                  ),
                                ),
                              ),
                            )
                          ],
                        ),
                      ),
                      _buildToolbar(),
                    ],
                  );
                }),
      ),
    );
  }

  Widget _buildToolbar() {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 32, vertical: 16),
      child: Column(
        children: [
          Text(nameOnTopbar,
              style: rubikBold.copyWith(color: Colors.white, fontSize: 20)),
          const SizedBox(
            height: 12,
          ),
          if (state[DB_KEY_STATUS] == CS_PICKED ||
              state[DB_KEY_STATUS] == CS_ENDED)
            WidgetTimer(
              builder: () {
                return Text("""AppUtils.stringDuration(Duration(
                        milliseconds: (DateTime.now().millisecondsSinceEpoch -
                                state[DB_KEY_PICKED_AT]!)
                            .toInt()))""",
                    style:
                        rubikBold.copyWith(color: Colors.white, fontSize: 32));
              },
            )
          else
            Text('00:00:00',
                style: rubikBold.copyWith(color: Colors.white, fontSize: 32)),
          const SizedBox(
            height: 16,
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: [
              ElevatedButton(
                style: ElevatedButton.styleFrom(
                    shape: const CircleBorder(),
                    padding: const EdgeInsets.all(12),
                    backgroundColor: Colors.white),
                onPressed: _onToggleMute,
                child: Icon(muted ? Icons.mic_off : Icons.mic,
                    color: Colors.black, size: 20),
              ),
              ElevatedButton(
                style: ElevatedButton.styleFrom(
                    shape: const CircleBorder(),
                    padding: const EdgeInsets.all(20),
                    backgroundColor: Colors.red),
                child:
                    const Icon(Icons.call_end, color: Colors.white, size: 35),
                onPressed: () {
                  Get.back();
                },
              ),
              Opacity(
                opacity: 0,
                child: ElevatedButton(
                  style: ElevatedButton.styleFrom(
                      shape: const CircleBorder(),
                      padding: const EdgeInsets.all(12),
                      backgroundColor: Colors.white),
                  child: const Icon(Icons.switch_camera,
                      color: Colors.black, size: 20),
                  onPressed: () {},
                ),
              ),
            ],
          ),
        ],
      ),
    );
  }
}

==> _private_chat-0.0.5.tar.gz: lib/call/widgets/video_call_page.dart <==
import 'dart:async';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:agora_rtc_engine/agora_rtc_engine.dart';
import 'package:get/get.dart';
import 'package:_private_chat/utils/utils.dart';
import 'package:_private_chat/widgets/widget_timer.dart';
import 'package:_private_core/_private_core.dart';
import '../call_helper.dart';
import 'widget_call_coming.dart';

class WidgetVideoCall extends StatefulWidget {
  final CallStatus status;

  final Call? call;

  final String? receiverId;
  final String? receiverName;
  final String? receiverPic;
  final String? receiverUserType;
  final String? conversationId;
  final String? conversationType;

  const WidgetVideoCall({
    Key? key,
    required this.status,
    this.call,
    this.receiverId,
    this.receiverName,
    this.receiverPic,
    this.receiverUserType,
    this.conversationId,
    this.conversationType,
  }) : super(key: key);

  @override
  WidgetVideoCallState createState() => WidgetVideoCallState();
}

class WidgetVideoCallState extends State<WidgetVideoCall> {
  late final RtcEngine _engine = createAgoraRtcEngine();
  Call? call;
  bool isInitialized = false;
  bool muted = false;
  bool _isAlreadyEnd = false;
  final _users = <int>[];
  Map<String, dynamic> state = {};

  late CallStatus status = widget.status;

  String get nameOnTopbar =>
      '${isCallComing ? call!.callerName : call!.receiverName}';

  bool get isCallComing => call?.callerId != loggedFirebaseId;

  @override
  void initState() {
    super.initState();
    _initialize();
  }

  Future<void> _initialize() async {
    setState(() => isInitialized = false);
    if (status == CallStatus.incomingCall) {
      if (widget.call == null) {
        Get.back();
        return;
      }
      call = widget.call!;
    } else {
      call = await CallHelper.dial(
        receiverName: widget.receiverName ?? '',
        receiverId: widget.receiverId ?? '',
        receiverPic: widget.receiverPic,
        receiverUserType: widget.receiverUserType ?? '',
        data: {
          'conversationId': widget.conversationId,
          'conversationType': widget.conversationType,
        },
      );
      if (call == null) {
        throw Exception('Call infos is null!');
      }
      await _engine.initialize(RtcEngineContext(
        appId: call!.agoraAppId!,
      ));

      appDebugPrint('Call info: ${call!.toJson()}');

      _engine.registerEventHandler(RtcEngineEventHandler(
        onError: (ErrorCodeType err, String msg) {
          appDebugPrint('[EventHandler] onError: $msg');
          if (!isCallComing) {}
        },
        onJoinChannelSuccess: (RtcConnection connection, int elapsed) {
          appDebugPrint(
              '[EventHandler] onJoinChannel: ${connection.channelId}, uid: ${connection.localUid}');
          if (!isCallComing) {}
        },
        onLeaveChannel: (RtcConnection connection, RtcStats stats) {
          appDebugPrint('[EventHandler] onLeaveChannel: $stats');
          _users.clear();
        },
        onUserJoined: (RtcConnection connection, int remoteUid, int elapsed) {
          appDebugPrint('[EventHandler] userJoined: $remoteUid');
          setState(() {
            _users.add(remoteUid);
          });
          if (!isCallComing) {
            CallHelper.updateCallState(call!.channelId,
                status: CS_PICKED,
                pickedAt: DateTime.now().millisecondsSinceEpoch);
          }
        },
        onUserOffline: (RtcConnection connection, int remoteUid,
            UserOfflineReasonType reason) {
          appDebugPrint('[EventHandler] userOffline: $remoteUid');
          setState(() {
            _users.remove(remoteUid);
          });
          Get.back();
          // if (isalreadyendedcall == false) {
          //   CallHelper.onEndCall(widget.call);
          // }
        },
        onFirstRemoteVideoFrame: (RtcConnection connection, int remoteUid,
            int width, int height, int elapsed) {
          appDebugPrint(
              '[EventHandler] firstRemoteVideo: $remoteUid ${width}x $height');
        },
      ));

      await _engine.enableVideo();
      // await _engine.setVideoEncoderConfiguration(
      //   const VideoEncoderConfiguration(
      //     dimensions: VideoDimensions(width: 640, height: 360),
      //     frameRate: 15,
      //     bitrate: 0,
      //   ),
      // );
      await _engine.startPreview();

      await _engine.joinChannel(
        token: call!.channelToken!,
        channelId: call!.channelId!,
        uid: 0,
        options: const ChannelMediaOptions(
          channelProfile: ChannelProfileType.channelProfileLiveBroadcasting,
          clientRoleType: ClientRoleType.clientRoleBroadcaster,
        ),
      );
    }
    setState(() => isInitialized = true);
  }

  void _onToggleMute() {
    setState(() => muted = !muted);
    _engine.muteLocalAudioStream(muted);
  }

  void _onEnd() {
    if (_isAlreadyEnd) {
    } else if (state[DB_KEY_STATUS] == CS_CANCELLED) {
    } else if (state[DB_KEY_STATUS] != CS_PICKED &&
        state[DB_KEY_STATUS] != CS_ENDED) {
      CallHelper.endCall(call: call!, status: CS_CANCELLED);
    } else {
      CallHelper.endCall(call: call!, status: CS_ENDED);
    }
    _users.clear();
    _engine.leaveChannel();
    _engine.release();
    _isAlreadyEnd = true;
  }

  @override
  void dispose() {
    _onEnd();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      color: Colors.transparent,
      child: Container(
        margin: const EdgeInsets.all(16),
        clipBehavior: Clip.hardEdge,
        padding: const EdgeInsets.all(5),
        decoration: BoxDecoration(
            color: hexColor('#E38F3E'),
            borderRadius: BorderRadius.circular(16)),
        child: !isInitialized
            ? Container(
                decoration: BoxDecoration(
                    color: Colors.white,
                    borderRadius: BorderRadius.circular(16)),
                child: const Center(child: CupertinoActivityIndicator()))
            : StreamBuilder<DocumentSnapshot>(
                stream: CallHelper.callStateStream(call!.channelId),
                builder: (_, snapshot) {
                  if (snapshot.data == null || snapshot.data!.exists == false) {
                    return const Center(child: CupertinoActivityIndicator());
                  }
                  Map<String, dynamic> doc =
                      snapshot.data!.data() as Map<String, dynamic>;
                  state = doc;
                  // appDebugPrint(
                  //     '[EventHandler] doc: ${call!.eventUniqueId} $doc');

                  if (doc[DB_KEY_STATUS] == CS_ENDED) {
                    Get.back();
                  } else if (doc[DB_KEY_STATUS] == CS_CANCELLED &&
                      isCallComing) {
                    Get.back();
                  } else if (doc[DB_KEY_STATUS] == CS_REJECTED &&
                      !isCallComing) {
                    Get.back();
                  }

                  if (doc[DB_KEY_STATUS] == CS_CALLING && isCallComing) {
                    return WidgetCallComming(
                      name: nameOnTopbar,
                      onCancel: () {
                        Get.back();
                        CallHelper.endCall(call: call!, status: CS_REJECTED);
                      },
                      onAccept: () async {
                        // bool isgranted = await CallHelper.permissions();
                        // if (isgranted) {
                        setState(() {
                          status = CallStatus.calling;
                        });
                        _initialize();
                        // } else {
                        //   Get.back();
                        //   CallHelper.endCall(call: call!, status: CS_REJECTED);
                        // }
                      },
                    );
                  }
                  return Column(
                    children: [
                      Expanded(
                        child: Stack(
                          alignment: Alignment.topCenter,
                          children: [
                            if (_users.isNotEmpty &&
                                doc[DB_KEY_STATUS] == CS_PICKED)
                              ClipRRect(
                                borderRadius: BorderRadius.circular(16),
                                child: AgoraVideoView(
                                  controller: VideoViewController.remote(
                                    rtcEngine: _engine,
                                    canvas: VideoCanvas(uid: _users.first),
                                    connection: RtcConnection(
                                        channelId: call!.channelId!),
                                    useFlutterTexture: true,
                                    useAndroidSurfaceView: false,
                                  ),
                                ),
                              )
                            else
                              Container(
                                decoration: BoxDecoration(
                                    color: Colors.white,
                                    borderRadius: BorderRadius.circular(16)),
                                width: Get.width,
                                height: Get.height,
                              ),
                            Positioned(
                              bottom: 20,
                              right: 20,
                              child: SizedBox(
                                width: Get.width / 8,
                                child: AspectRatio(
                                  aspectRatio: 1.5,
                                  child: ClipRRect(
                                    borderRadius: BorderRadius.circular(8),
                                    child: AgoraVideoView(
                                      controller: VideoViewController(
                                        rtcEngine: _engine,
                                        canvas: const VideoCanvas(uid: 0),
                                        useFlutterTexture: true,
                                        useAndroidSurfaceView: false,
                                      ),
                                    ),
                                  ),
                                ),
                              ),
                            )
                          ],
                        ),
                      ),
                      _buildToolbar(),
                    ],
                  );
                }),
      ),
    );
  }

  Widget _buildToolbar() {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 32, vertical: 16),
      child: Column(
        children: [
          Text(nameOnTopbar,
              style: rubikBold.copyWith(color: Colors.white, fontSize: 20)),
          const SizedBox(
            height: 12,
          ),
          if (state[DB_KEY_STATUS] == CS_PICKED ||
              state[DB_KEY_STATUS] == CS_ENDED)
            WidgetTimer(
              builder: () {
                return Text("""AppUtils.stringDuration(Duration(
                        milliseconds: (DateTime.now().millisecondsSinceEpoch -
                                state[DB_KEY_PICKED_AT]!)
                            .toInt()))""",
                    style:
                        rubikBold.copyWith(color: Colors.white, fontSize: 32));
              },
            )
          else
            Text('00:00:00',
                style: rubikBold.copyWith(color: Colors.white, fontSize: 32)),
          const SizedBox(
            height: 16,
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: [
              ElevatedButton(
                style: ElevatedButton.styleFrom(
                    shape: const CircleBorder(),
                    padding: const EdgeInsets.all(12),
                    backgroundColor: Colors.white),
                onPressed: _onToggleMute,
                child: Icon(muted ? Icons.mic_off : Icons.mic,
                    color: Colors.black, size: 20),
              ),
              ElevatedButton(
                style: ElevatedButton.styleFrom(
                    shape: const CircleBorder(),
                    padding: const EdgeInsets.all(20),
                    backgroundColor: Colors.red),
                child:
                    const Icon(Icons.call_end, color: Colors.white, size: 35),
                onPressed: () {
                  Get.back();
                },
              ),
              Opacity(
                opacity: 0,
                child: ElevatedButton(
                  style: ElevatedButton.styleFrom(
                      shape: const CircleBorder(),
                      padding: const EdgeInsets.all(12),
                      backgroundColor: Colors.white),
                  child: const Icon(Icons.switch_camera,
                      color: Colors.black, size: 20),
                  onPressed: () {},
                ),
              ),
            ],
          ),
        ],
      ),
    );
  }
}
